###############################################################################
###############################################################################
# PROGRAM:
# auxiliar-functions-simulation.R
# 
# USE:
# Define auxiliar functions to be used in the simulation study for the ZMPS model
#     
# AUTHOR:
# Wesley Bertoli (wbsilva@utfpr.edu.br)
#
# DATE:
# January 22, 2021
###############################################################################
###############################################################################

# Setting parameter values

gen.parms <- function (stp) {
  
  if (stp[2] == 1) {
    
    b0 <- 1.5
    b1 <- 3
    g0 <- -1
    g1 <- -1
    
    if (stp[3] == 2) g1 <- .5
    if (stp[3] == 3) b1 <- -1.5    
    if (stp[3] == 4) { b1 <- -1.5; g1 <- .5 }
     
  } else {
    
    b0 <- -1
    b1 <- 1
    g0 <- .5
    g1 <- .5
    
    if (stp[3] == 2) { g0 <- 1.5; g1 <- -1 }
    if (stp[3] == 3) b1 <- -1.5    
    if (stp[3] == 4) { b1 <- -1.5; g0 <- 1.5; g1 <- -1 }
    
  }
  c(b0, b1, g0, g1, stp[1])
  
}
gen.parms <- cmpfun(f = gen.parms)

###############################################################################
###############################################################################

# Simulation process

to.run <- function (i, link, parms) {
  
  k   <- 1
  err <- T
  
  x <- runif(n = parms[5])
  y <- numeric(length = parms[5])
  
  while (err) {
    
    e1 <- exp(x = h(a = exp(x = parms[1] + parms[2] * x), log = T))
    e2 <- make.link(link = link)$linkinv(eta = parms[3] + parms[4] * x) 
    
    for (j in 1:parms[5]) y[j] <- SeqSea.d(a = e1[j], b = e2[j])
    
    aux.data <- list(n = length(x = y), y = y, X1 = cbind(a = 1, b = x), X2 = cbind(a = 1, b = x), link = link)
    
    fit <- try(expr = RwM(data = aux.data), silent = T)
    print(fit)
    err <- ifelse(test = !inherits(x = fit, what = 'try-error'), yes = F, no = T)
    
    k <- k + 1
    
  }
  
  list(fit$chain[seq(from = 1, to = nrow(x = fit$chain), by = 10),], aux.data)
  
}
to.run <- cmpfun(f = to.run)

###############################################################################
###############################################################################