###############################################################################
###############################################################################
# PROGRAM:
# estimation-rqr.R
# 
# USE:
# Estimation of the Randomized Quantile Residuals (RQRs) - ZMPS model
#     
# AUTHOR:
# Wesley Bertoli (wbsilva@utfpr.edu.br)
#
# DATE:
# January 22, 2021
###############################################################################
###############################################################################

# Clear workspace

rm(list = ls())

###############################################################################
###############################################################################

# Loading required packages

invisible(x = lapply(X = c('compiler', 'parallel'), FUN = require, character.only = T))

###############################################################################
###############################################################################

# Enabling just-in-time compilation

enableJIT(level = 3)

###############################################################################
###############################################################################

# Define the required functions and objects in the workspace

source(file = 'Codes/auxiliar-functions.R')
source(file = 'Codes/rqr-specific-functions.R')

load(file = 'RData/zmps_fitted_model.RData')

###############################################################################
###############################################################################

# Starting a cluster with specific number of cores

n.cores <- detectCores() - 1

cl <- makeCluster(spec = n.cores)

###############################################################################
###############################################################################

# Export required packages to cluster

invisible(x = clusterEvalQ(cl   = cl,
                           expr = { lapply(X   = c('MASS', 'mvtnorm', 'numDeriv'),
                                           FUN = require, character.only = T) }))

###############################################################################
###############################################################################

# Exporting functions to cluster

fun.list <- c('dztps', 'dzmps', 'pzmps', 'h', 'loglike.b1', 'loglike.b2', 'logpost.b1',
              'logpost.b2', 'lin.pred', 'makePD', 'RwM', 'SeqSea.d', 'rqr.fun', 'to.run')

clusterExport(cl = cl, varlist = fun.list)

###############################################################################
###############################################################################

# Estimating the RQRs

set.seed(seed = aux.data$n, kind = "L'Ecuyer-CMRG")

result <- clusterApplyLB(cl = cl, x = 1:aux.data$n, fun = to.run, data = aux.data, lp = lp)
rm(list = setdiff(x = ls(), y = 'result'))

stopCluster(cl = cl)

###############################################################################
###############################################################################

# Saving the results

save.image('RData/results_rqr.RData')

###############################################################################
###############################################################################